/*----------------------------------------------------------------------------
    S.M.E.L.T. : Small Musically Expressive Laptop Toolkit

    Copyright (c) 2007 Rebecca Fiebrink and Ge Wang.  All rights reserved.
      http://smelt.cs.princeton.edu/
      http://soundlab.cs.princeton.edu/

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
    U.S.A.
-----------------------------------------------------------------------------*/

//-----------------------------------------------------------------------------
// name: motion-sense.ck
// desc: this program attempts to use the Sudden Motion Sensor (SMS) on
//       newer apple laptops, and polls 3-axis tilt.
//
// authors: Spencer Salazar
// commented by: Rebecca Fiebrink and Ge Wang
//
// requires: SMS-enabled powerbook or macbook running OS X (10.4.8 or higher)
//
// to run (in command line chuck):
//     %> chuck motion-sense.ck
//
// to run (in miniAudicle):
//     (make sure VM is started, add the thing)
//-----------------------------------------------------------------------------

// instantiate a HidIn object
HidIn hi;
HidMsg msg;

// open tilt sensor
if( !hi.openTiltSensor() )
{
    <<< "tilt sensor unavailable", "" >>>;
    me.exit();
}

// print
<<< "tilt sensor ready", "" >>>;

// infinite while loop
while( true )
{
    // poll the tilt sensor, expect to get back 3 element array of ints
    // (9 for now means accelerometer, 0 selects 0th accelerometer)
    hi.read( 9, 0, msg );
    // print results, note these are "absolute" tilt in each axis
    <<< msg.x, msg.y, msg.z >>>;
    // advance time
    100::ms => now;
}
